// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

import Qt 4.7
import mapplugin 1.0

Flickable {
    overShoot: false
    contentWidth: Math.max(view.width, width)
    contentHeight: Math.max(view.height, height)

    property string file : ""
    property alias highlight : mapHighlight
    property alias model: mapModel

    MapModel {
        id: mapModel
        source: file
    }

    MapView {
        id: view
        model: mapModel
        blockSize: editor.blockSize

        function pointForPosition(xpos, ypos) {
            return { x: Math.floor(xpos/blockSize), y: Math.floor(ypos/blockSize) };
        }
        function indexForPosition(xpos, ypos) {
            return Qt.point(Math.floor(xpos/blockSize), Math.floor(ypos/blockSize));
        }

        MouseArea {
            anchors.fill: parent
            acceptedButtons: Qt.LeftButton | Qt.RightButton | Qt.MidButton
            hoverEnabled: true

            onClicked: {
                if (editor.state === "editObjects" || editor.state === "chooseType") {
                    if (mouse.button === Qt.LeftButton) {
                        highlight.index = view.indexForPosition(mouse.x, mouse.y);
                        var i = view.pointForPosition(mouse.x, mouse.y);
                        highlight.indexX = i.x;
                        highlight.indexY = i.y;
                    } else if (mouse.button === Qt.MidButton) {
                        mapModel.setType(view.indexForPosition(mouse.x, mouse.y), 0)
                    } else if (mouse.button === Qt.RightButton) {
                        if (highlight.recent.length > 0)
                            mapModel.setType(view.indexForPosition(mouse.x, mouse.y), highlight.recent[highlight.recent.length - 1])
                    }
                }
            }

            function drawHelper(mouse) {
                var index = view.indexForPosition(mouse.x, mouse.y);
                if (mouse.buttons === Qt.LeftButton) {
                    mapModel.setType(index, 0);
                } else if (mouse.buttons === Qt.RightButton) {
                    mapModel.setType(index, 2);
                } else if (mouse.buttons === Qt.MidButton) {
                    mapModel.setType(index, 3);
                }
            }

            onPressed: {
                if (editor.state === "draw") {
                    drawHelper(mouse);
                }
            }

            onPositionChanged: {
                if (editor.state === "draw") {
                    drawHelper(mouse);
                    return;
                }

                var type = mapModel.type(view.indexForPosition(mouseX, mouseY));
                if (type <= 3) {
                    mapTooltip.visible = false;
                    return;
                }
                mapTooltip.x = mouseX + 20;
                mapTooltip.y = mouseY + 20;
                mapTooltip.visible = true;
                mapTooltip.text = "unknown";
                for (var i = 0; i < typeSelector.objectsModel.count; ++i) {
                    var o = typeSelector.objectsModel.get(i);
                    if (o.type == type) {
                        mapTooltip.text = o.name;
                    }
                }
            }
        }

        TextButton {
            id: mapTooltip
            visible: false
        }

        Item {
            id: mapHighlight
            x: indexX ? indexX * blockSize : 0
            y: indexY ? indexY * blockSize : 0
            width: blockSize
            height: blockSize
            visible: index !== undefined

            property variant index : undefined
            property variant indexX : undefined
            property variant indexY : undefined
            property variant recent : []

            Rectangle {
                anchors.centerIn: parent
                width: parent.width + border.width
                height: parent.height + border.width
                color: "transparent"
                border.width: 4
                border.color: uiHighlightColor
            }

            TextButton {
                anchors.right: parent.left
                anchors.bottom: parent.top
                text: "Choose"

                onClicked: editor.state = "chooseType";
            }

            TextButton {
                anchors.left: parent.right
                anchors.bottom: parent.top
                text: "Clear"

                onClicked: mapModel.setType(parent.index, 0);
            }

            Rectangle {
                anchors.right: parent.left
                anchors.top: parent.bottom
                width: recentSelector.width + mediumMargin
                height: recentSelector.height + mediumMargin
                border.color: uiBorderColor
                border.width: 1
                visible: mapHighlight.recent.length > 0

                Row {
                    id: recentSelector
                    anchors.centerIn: parent
                    Repeater {
                        model: recentCount
                        StarGuardObject {
                            type: mapHighlight.recent.length > index ? mapHighlight.recent[index] : 0
                            visible: mapHighlight.recent.length > index
                            width: blockSize
                            height: blockSize

                            MouseArea {
                                anchors.fill: parent
                                onClicked: mapModel.setType(mapHighlight.index, type)
                            }
                        }
                    }
                }

            }
        }
    }
}
