// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

import Qt 4.7
import mapplugin 1.0

Rectangle {
    id: typeSelector
    color: uiBackgroundColor
    width: selector.cellWidth*6 + 11
    border.color: uiHighlightColor
    border.width: 2

    property variant objectsModel: ObjectList { }

    Component {
        id: objectDelegate
        Rectangle {
            width: 50
            height: 50
            color: "transparent"
            border.width: 1
            border.color: uiBorderColor
            visible: model.hidden ? false : true
            StarGuardObject {
                type: model.type
                width: parent.width - 2*smallMargin
                height: parent.height - 2*smallMargin
                anchors.centerIn: parent
            }
            MouseArea {
                id: mouseArea
                anchors.fill: parent
                hoverEnabled: true
                onClicked: {
                    map.model.setType(map.highlight.index, type);
                    var newRecent = map.highlight.recent;
                    if (newRecent[newRecent.length-1] != type) {
                        newRecent.push(type);
                        if (newRecent.length > recentCount)
                            newRecent.shift();
                        map.highlight.recent = newRecent;
                    }
                    editor.state = "editObjects";
                }
                onPositionChanged: {
                    objectTooltip.text = model.name
                    var pos = typeSelector.mapFromItem(mouseArea, mouse.x, mouse.y);
                    objectTooltip.x = pos.x - objectTooltip.width - mediumMargin
                    objectTooltip.y = pos.y + mediumMargin
                    objectTooltip.visible = true
                }
                onExited: objectTooltip.visible = false
            }
        }
    }

    TextButton {
        id: closeButton
        text: "Close"
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: mediumMargin

        onClicked: editor.state = "editObjects";
    }

    GridView {
        id: selector
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: closeButton.bottom
        anchors.bottom: parent.bottom
        anchors.margins: mediumMargin
        clip: true

        cellWidth: 50
        cellHeight: 50

        model: objectsModel
        delegate: objectDelegate
    }

    TextButton {
        id: objectTooltip
        visible: false
    }
}
